@@ -14,9 +14,12 @@
  * - /dev/audio (if the admins install GTK and GNOME on our Sparcs...)
  * - other sound systems
  * - you name it...
+ * - Animated ramping of mixer sliders to preset values
+ * - Load Settings button, several for presets ?
  * - more configuration
  *
  * Copyright (C) 1998 Jens Ch. Restemeier <jchrr@hrz.uni-bielefeld.de>
+ * Config dialog added by Matt Martin <Matt.Martin@ieee.org>, Sept 1999
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -57,6 +60,8 @@
 static void about_cb (GtkWidget *widget, void *data);
 static void options_cb (GtkWidget *widget, void *data);
 static void quit_cb (GtkWidget *widget, void *data);
+static void config_cb (GtkWidget *widget, void *data);
+void fill_in_device_guis(GtkWidget *notebook);
 
 static void get_device_config(void);
 static void put_device_config(void);
@@ -71,17 +76,29 @@ void open_dialog(void);
  * Gnome info:
  */
 GtkWidget *app;
+GtkWidget *configwin;
+GtkWidget  *slidernotebook;
+
+typedef struct {
+	gboolean set_mixer_on_start;
+	gboolean hide_menu;
+	gboolean use_icons;
+	gboolean use_labels;
+} mixerprefs;
+
+mixerprefs prefs={FALSE,FALSE,TRUE,TRUE};
 
 /* Menus */
 static GnomeUIInfo help_menu[] = {
-    GNOMEUIINFO_ITEM_STOCK(N_("_About"), NULL, about_cb,
-                           GNOME_STOCK_MENU_ABOUT),
-    GNOMEUIINFO_END
+	GNOMEUIINFO_ITEM_STOCK(N_("_About"), NULL, about_cb,
+			       GNOME_STOCK_MENU_ABOUT),
+	GNOMEUIINFO_END
 };
  
 static GnomeUIInfo program_menu[] = {
-    GNOMEUIINFO_MENU_EXIT_ITEM(quit_cb, NULL),
-    GNOMEUIINFO_END
+	GNOMEUIINFO_ITEM_STOCK(N_("_Preferences..."),NULL,config_cb,GNOME_STOCK_MENU_PREF),
+	GNOMEUIINFO_MENU_EXIT_ITEM(quit_cb, NULL),
+	GNOMEUIINFO_END
 };      
 
 static GnomeUIInfo main_menu[] = {
@@ -130,16 +147,17 @@ typedef struct channel_info {
 
 GList *devices;
 
-int mode=0, mode_norestore=0, mode_initonly=0, mode_nosave=0, num_mixers;
+int mode=0, mode_norestore=0, mode_initonly=0, mode_nosave=0, num_mixers, mode_restore=0;
 #define M_NORESTORE	1
 #define M_INITONLY 	2
 #define M_NOSAVE	4
 
 static const struct poptOption options[] = {
-  {"norestore", 'r', POPT_ARG_NONE, &mode_norestore, 0, N_("don't restore mixer-settings from configuration"), NULL},
-  {"initonly", 'i', POPT_ARG_NONE, &mode_initonly, 0, N_("initialise the mixer(s) from stored configuration and exit"), NULL},
-  {"nosave", 's', POPT_ARG_NONE, &mode_nosave, 0, N_("don't save (modified) mixer-settings into configuration"), NULL},
-  {NULL, '\0', 0, NULL, 0}
+	{"norestore", 'r', POPT_ARG_NONE, &mode_norestore, 0, N_("don't restore mixer-settings from configuration"), NULL},
+	{"restore", 'R', POPT_ARG_NONE, &mode_restore, 0, N_("restore mixer-settings from configuration"), NULL},
+	{"initonly", 'i', POPT_ARG_NONE, &mode_initonly, 0, N_("initialise the mixer(s) from stored configuration and exit"), NULL},
+	{"nosave", 's', POPT_ARG_NONE, &mode_nosave, 0, N_("don't save (modified) mixer-settings into configuration"), NULL},
+	{NULL, '\0', 0, NULL, 0}
 };
 
 /*
@@ -156,34 +174,34 @@ const char *device_names[]  = SOUND_DEVICE_NAMES;
 #define GNOME_STOCK_PIXMAP_BLANK NULL
 #endif
 const char *device_pixmap[] = {
-   GNOME_STOCK_PIXMAP_VOLUME,               /* Master Volume */
-   GNOME_STOCK_PIXMAP_BLANK,                /* Bass */
-   GNOME_STOCK_PIXMAP_BLANK,                /* Treble */
-   GNOME_STOCK_PIXMAP_BLANK,                /* Synth */
-   GNOME_STOCK_PIXMAP_BLANK,                /* PCM */
-   GNOME_STOCK_PIXMAP_VOLUME,               /* Speaker */
-   GNOME_STOCK_PIXMAP_LINE_IN,              /* Line In */
-   GNOME_STOCK_PIXMAP_MIC,                  /* Microphone */
-   GNOME_STOCK_PIXMAP_CDROM,                /* CD-Rom */
-   GNOME_STOCK_PIXMAP_BLANK,                /* Recording monitor ? */
-   GNOME_STOCK_PIXMAP_BLANK,                /* ALT PCM */
-   GNOME_STOCK_PIXMAP_BLANK,                /* Rec Level? */
-   GNOME_STOCK_PIXMAP_BLANK,                /* In Gain */
-   GNOME_STOCK_PIXMAP_BLANK,                /* Out Gain */
-   GNOME_STOCK_PIXMAP_LINE_IN,              /* Aux 1 */
-   GNOME_STOCK_PIXMAP_LINE_IN,              /* Aux 2 */
-   GNOME_STOCK_PIXMAP_LINE_IN,              /* Line */
-   GNOME_STOCK_PIXMAP_BLANK,                /* Digital 1 ? */
-   GNOME_STOCK_PIXMAP_BLANK,                /* Digital 2 ? */
-   GNOME_STOCK_PIXMAP_BLANK,                /* Digital 3 ? */
-   GNOME_STOCK_PIXMAP_BLANK,                /* Phone in */
-   GNOME_STOCK_PIXMAP_BLANK,                /* Phone Out */
-   GNOME_STOCK_PIXMAP_BLANK,                /* Video */
-   GNOME_STOCK_PIXMAP_BLANK,                /* Radio */
-   GNOME_STOCK_PIXMAP_BLANK,                /* Monitor (usually mic) vol */
-   GNOME_STOCK_PIXMAP_BLANK,                /* 3d Depth/space param */
-   GNOME_STOCK_PIXMAP_BLANK,                /* 3d center param */
-   GNOME_STOCK_PIXMAP_BLANK                 /* Midi */
+	GNOME_STOCK_PIXMAP_VOLUME,               /* Master Volume */
+	GNOME_STOCK_PIXMAP_BLANK,                /* Bass */
+	GNOME_STOCK_PIXMAP_BLANK,                /* Treble */
+	GNOME_STOCK_PIXMAP_BLANK,                /* Synth */
+	GNOME_STOCK_PIXMAP_BLANK,                /* PCM */
+	GNOME_STOCK_PIXMAP_VOLUME,               /* Speaker */
+	GNOME_STOCK_PIXMAP_LINE_IN,              /* Line In */
+	GNOME_STOCK_PIXMAP_MIC,                  /* Microphone */
+	GNOME_STOCK_PIXMAP_CDROM,                /* CD-Rom */
+	GNOME_STOCK_PIXMAP_BLANK,                /* Recording monitor ? */
+	GNOME_STOCK_PIXMAP_BLANK,                /* ALT PCM */
+	GNOME_STOCK_PIXMAP_BLANK,                /* Rec Level? */
+	GNOME_STOCK_PIXMAP_BLANK,                /* In Gain */
+	GNOME_STOCK_PIXMAP_BLANK,                /* Out Gain */
+	GNOME_STOCK_PIXMAP_LINE_IN,              /* Aux 1 */
+	GNOME_STOCK_PIXMAP_LINE_IN,              /* Aux 2 */
+	GNOME_STOCK_PIXMAP_LINE_IN,              /* Line */
+	GNOME_STOCK_PIXMAP_BLANK,                /* Digital 1 ? */
+	GNOME_STOCK_PIXMAP_BLANK,                /* Digital 2 ? */
+	GNOME_STOCK_PIXMAP_BLANK,                /* Digital 3 ? */
+	GNOME_STOCK_PIXMAP_BLANK,                /* Phone in */
+	GNOME_STOCK_PIXMAP_BLANK,                /* Phone Out */
+	GNOME_STOCK_PIXMAP_BLANK,                /* Video */
+	GNOME_STOCK_PIXMAP_BLANK,                /* Radio */
+	GNOME_STOCK_PIXMAP_BLANK,                /* Monitor (usually mic) vol */
+	GNOME_STOCK_PIXMAP_BLANK,                /* 3d Depth/space param */
+	GNOME_STOCK_PIXMAP_BLANK,                /* 3d center param */
+	GNOME_STOCK_PIXMAP_BLANK                 /* Midi */
 };
 
 /*
@@ -211,9 +229,131 @@ GtkWidget *make_slider_mixer(channel_info *ci);
 
 static void error_close_cb(void)
 {
-    g_error("No mixers found.");
+        g_error("No mixers found.");
 }
 
+static void apply_cb(GtkWidget *widget, void *data)
+{       
+        /* This is a sloppy way to re-draw the mixers */
+        GList *d;
+
+        put_gui_config();
+
+	gtk_widget_hide(slidernotebook);
+	/* Assumes that the number of devices is static... */
+	for (d=devices; d; d=d->next) 
+		gtk_notebook_remove_page(GTK_NOTEBOOK(slidernotebook),0);
+	gtk_widget_show(slidernotebook);
+
+	fill_in_device_guis(slidernotebook);
+	
+}
+
+static void cancel_cb(GtkWidget *widget, void *data)
+{       
+        configwin=NULL;
+}
+
+static void bool_changed_cb(GtkWidget *widget, gboolean *data)
+{
+	if( *data )
+		*data = FALSE;
+	else        
+		*data = TRUE;
+	gnome_property_box_changed(GNOME_PROPERTY_BOX(configwin));
+}
+
+GtkWidget *make_config_win()
+{
+}
+
+GtkWidget *optpage()
+{
+	GtkWidget *start_frame, *gui_frame;
+	GtkWidget *ubervbox;
+	GtkWidget *vbox, *init_start, *menu_hide, *temp;
+
+	ubervbox = gtk_vbox_new(TRUE, 0);
+
+	start_frame = gtk_frame_new(_("On startup"));
+	gtk_container_border_width(GTK_CONTAINER(start_frame), GNOME_PAD_SMALL);
+    
+	vbox = gtk_vbox_new(TRUE, 0);
+
+	/* Set on start */
+	init_start = gtk_check_button_new_with_label(_("Restore saved mixer levels on startup"));
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(init_start), \
+				     prefs.set_mixer_on_start);
+	gtk_signal_connect(GTK_OBJECT(init_start), "clicked",
+			   GTK_SIGNAL_FUNC(bool_changed_cb), &prefs.set_mixer_on_start);
+
+	menu_hide = gtk_check_button_new_with_label(_("Hide menu on start"));
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(menu_hide), \
+				     prefs.hide_menu);
+	gtk_signal_connect(GTK_OBJECT(menu_hide), "clicked",
+			   GTK_SIGNAL_FUNC(bool_changed_cb), &prefs.hide_menu);
+
+	gtk_box_pack_start_defaults(GTK_BOX(vbox), init_start);
+	/*	gtk_box_pack_start_defaults(GTK_BOX(vbox), menu_hide);*/
+
+	gtk_widget_show_all(vbox);
+
+
+	gtk_container_add(GTK_CONTAINER(start_frame), vbox);
+	gtk_container_add(GTK_CONTAINER(ubervbox), start_frame);
+
+	gui_frame = gtk_frame_new(_("GUI"));
+	gtk_container_border_width(GTK_CONTAINER(gui_frame), GNOME_PAD_SMALL);
+
+	vbox = gtk_vbox_new(TRUE, 0);
+	temp = gtk_check_button_new_with_label(_("Use Mixer Icons"));
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), \
+				     prefs.use_icons);
+	gtk_signal_connect(GTK_OBJECT(temp), "clicked",
+			   GTK_SIGNAL_FUNC(bool_changed_cb), &prefs.use_icons);
+	gtk_box_pack_start_defaults(GTK_BOX(vbox), temp);
+	
+	
+	temp = gtk_check_button_new_with_label(_("Use Mixer Labels"));
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), \
+				     prefs.use_labels);
+	gtk_signal_connect(GTK_OBJECT(temp), "clicked",
+			   GTK_SIGNAL_FUNC(bool_changed_cb), &prefs.use_labels);
+	gtk_box_pack_start_defaults(GTK_BOX(vbox), temp);
+	
+	
+	gtk_container_add(GTK_CONTAINER(gui_frame), vbox);
+	
+	gtk_container_add(GTK_CONTAINER(ubervbox), gui_frame);
+	
+	gtk_widget_show_all(ubervbox);
+
+	return ubervbox;
+}
+
+void config_cb(GtkWidget *widget, void *data)
+{
+        GtkWidget *label;
+	if (!configwin) {
+	        configwin=gnome_property_box_new();
+		gtk_widget_realize(configwin);
+		label = gtk_label_new(_("Preferences"));
+		gtk_notebook_append_page(
+			GTK_NOTEBOOK(
+			GNOME_PROPERTY_BOX(configwin)->notebook), optpage(), label);
+		
+		gtk_signal_connect(GTK_OBJECT(configwin), "apply",
+				   GTK_SIGNAL_FUNC(apply_cb), NULL);
+		gtk_signal_connect(GTK_OBJECT(configwin), "destroy",
+				   GTK_SIGNAL_FUNC(cancel_cb), NULL);
+		gtk_signal_connect(GTK_OBJECT(configwin), "delete_event",
+				   GTK_SIGNAL_FUNC(cancel_cb), NULL);
+
+		gtk_widget_show_all(configwin);	
+    
+	};
+};
+
 int main(int argc, char *argv[]) 
 {
 	mode=0;
@@ -225,21 +365,30 @@ int main(int argc, char *argv[])
 
 	if(mode_nosave) mode |= M_NOSAVE;
 	if(mode_initonly) mode |= M_INITONLY;
-	if(mode_norestore) mode |= M_NORESTORE;
 
 	scan_devices();
 	if (devices) {
+		if (~mode & M_INITONLY) {
+		        get_gui_config();
+			/* Beweare boolean bastardization */
+			if (!prefs.set_mixer_on_start)
+				mode |= M_NORESTORE;
+			  
+		}
+		/* Command line always overrides */
+		if(mode_norestore) mode |= M_NORESTORE;
+		if(mode_restore) mode &= ~M_NORESTORE;
+
 		if (~mode & M_NORESTORE) {
 			get_device_config();
 			init_devices();
 		} 
 
 		if (~mode & M_INITONLY) {
-			get_gui_config();
 
 			open_dialog();
 
-			put_gui_config();
+			put_gui_config();/* Do we need this*/
 		}
 
 		if (~mode & M_NOSAVE) {
@@ -248,11 +397,11 @@ int main(int argc, char *argv[])
 		gnome_config_sync();
 		free_devices();
 	} else {
-	    GtkWidget *box;
-	    box = gnome_error_dialog("No mixers found.\nMake sure you have sound support compiled into the kernel.");
-	    gtk_signal_connect(GTK_OBJECT(box), "close",
-			       GTK_SIGNAL_FUNC(error_close_cb), NULL);
-	    gtk_main();
+		GtkWidget *box;
+		box = gnome_error_dialog("No mixers found.\nMake sure you have sound support compiled into the kernel.");
+		gtk_signal_connect(GTK_OBJECT(box), "close",
+				   GTK_SIGNAL_FUNC(error_close_cb), NULL);
+		gtk_main();
 	}
 	return 0;
 }
@@ -323,7 +472,7 @@ device_info *open_device(int num)
 	new_device->enabled_bitmask=new_device->devmask;	/* all enabled */
 	for (cnt=0; cnt<SOUND_MIXER_NRDEVICES; cnt++) {
 		if (new_device->devmask & (1<<cnt)) {
-		    unsigned long vol; // l: vol&0xff, r:(vol&0xff00)>>8
+			unsigned long vol; // l: vol&0xff, r:(vol&0xff00)>>8
 			res=ioctl(new_device->fd, MIXER_READ(cnt), &vol);
 		                                                
 			new_device->volume_left[cnt]=vol & 0xff;
@@ -374,7 +523,7 @@ GList *make_channels(device_info *device)
 			new_channel->device=device;
 			new_channel->channel=i;
 	 		new_channel->pixmap = g_strdup (device_pixmap[i]);
-			new_channel->title=strdup(device_labels[i]);
+			new_channel->title= g_strdup(device_labels[i]);
 			new_channel->passive=0;
 			channels=g_list_append(channels, new_channel);
 		}
@@ -411,7 +560,7 @@ void free_devices(void)
 
 void init_one_device(gpointer a, gpointer b)
 {
-    unsigned long vol;
+	unsigned long vol;
 	int c;
 	
 	device_info *info = (device_info *)a;
@@ -422,8 +571,8 @@ void init_one_device(gpointer a, gpointer b)
 			if (info->mute_bitmask & (1<<c)) {
 				vol=0;
 			} else {
-			    vol = info->volume_left[c];
-			    vol |= info->volume_right[c]<<8;
+				vol = info->volume_left[c];
+				vol |= info->volume_right[c]<<8;
 			}
 			ioctl(info->fd, MIXER_WRITE(c), &vol);
 		}
@@ -445,7 +594,9 @@ void get_one_device_config(gpointer a, gpointer b)
 	char name[255];
 	for (cc=0; cc<SOUND_MIXER_NRDEVICES; cc++) {
 		if (info->devmask & (1<<cc)) {
-			sprintf(name, "/gmix/%s_%s/mute=%s", info->info.id, device_names[cc], (info->mute_bitmask & (1<<cc))?"true":"false");
+			sprintf(name, "/gmix/%s_%s/mute=%s", info->info.id,\
+				device_names[cc],\
+				(info->mute_bitmask & (1<<cc))?"true":"false");
 			info->mute_bitmask&=~(1<<cc);
 			info->mute_bitmask|= gnome_config_get_bool(name) ? (1<<cc) : 0;
 			if (info->recmask & (1<<cc)) {
@@ -512,10 +663,18 @@ void put_device_config(void)
 
 void get_gui_config(void)
 {
+	prefs.set_mixer_on_start=gnome_config_get_bool("/gmix/startup/init");
+	prefs.hide_menu=gnome_config_get_bool("/gmix/gui/menu");
+	prefs.use_icons=gnome_config_get_bool("/gmix/gui/icons");
+	prefs.use_labels=gnome_config_get_bool("/gmix/gui/labels");
 }
 
 void put_gui_config(void)
 {
+	gnome_config_set_bool("/gmix/startup/init",prefs.set_mixer_on_start);
+	gnome_config_set_bool("/gmix/gui/menu",prefs.hide_menu);
+	gnome_config_set_bool("/gmix/gui/icons",prefs.use_icons);
+	gnome_config_set_bool("/gmix/gui/labels",prefs.use_labels);
 }
 
 /*
@@ -552,9 +711,103 @@ GtkWidget *make_slider_mixer(channel_info *ci)
 	return hbox;
 }
 
+void fill_in_device_guis(GtkWidget *notebook){
+	GList *d, *c;
+	GtkWidget *table, *spixmap;
+	int i,j;
+
+	i=0;
+	for (d=devices; d; d=d->next) {
+		device_info *di;
+		di=d->data;
+		j=0;
+		for (c=((device_info *)d->data)->channels;c;c=c->next) {
+			j+=2;
+		}
+
+		/* 
+		 * David: changed 7 to 8 for table rows (06/04/1999)
+		 */
+		table=gtk_table_new(i*2, 8, FALSE);
+		gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
+					 table, 
+					 gtk_label_new(di->info.name));
+		gtk_table_set_row_spacings (GTK_TABLE (table), 0);
+		gtk_table_set_col_spacings (GTK_TABLE (table), 0);
+		gtk_container_border_width (GTK_CONTAINER (table), 0);
+		gtk_widget_show (table);
+
+		for (c=((device_info *)d->data)->channels;c;c=c->next) {
+			GtkWidget *label, *mixer, *separator;
+			channel_info *ci;
+			ci=c->data;
+			if ((ci->pixmap)&&(prefs.use_icons))
+			{
+				spixmap = gnome_stock_pixmap_widget (app, ci->pixmap);
+				gtk_table_attach (GTK_TABLE (table), spixmap, i, i+1, 1, 2, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
+				gtk_widget_show (spixmap);
+			}
+			if (prefs.use_labels) {
+				label=gtk_label_new(ci->title);
+				gtk_misc_set_alignment (GTK_MISC(label), 0.1, 0.5);
+				gtk_table_attach (GTK_TABLE (table), label, i, i+1, 2, 3, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
+				gtk_widget_show(label);
+			}
+
+			mixer=make_slider_mixer(ci);
+			gtk_table_attach (GTK_TABLE (table), mixer, i, i+1,\
+					  3, 4, GTK_EXPAND | GTK_FILL,\
+					  GTK_EXPAND | GTK_FILL, 0, 0);
+			gtk_widget_show (mixer);
+
+			if (ci->device->stereodevs & (1<<ci->channel)) {
+				/* lock-button, only useful for stereo */
+				ci->lock = gtk_check_button_new_with_label(_("Lock"));
+				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ci->lock), (ci->device->lock_bitmask & (1<<ci->channel))!=0);
+				gtk_signal_connect (GTK_OBJECT (ci->lock), "toggled", (GtkSignalFunc) lock_cb, (gpointer)ci);
+				gtk_table_attach (GTK_TABLE (table), ci->lock, i, i+1, 4, 5, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
+				gtk_widget_show (ci->lock);
+			}
+			/*
+			 * recording sources
+			 */
+			if (ci->device->recmask & (1<<ci->channel)) {
+				ci->rec = gtk_check_button_new_with_label (_("Rec."));
+				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ci->rec), (ci->device->record_bitmask & (1<<ci->channel))!=0);
+				gtk_signal_connect (GTK_OBJECT (ci->rec), "toggled", (GtkSignalFunc) rec_cb, (gpointer)ci);
+				/* David: changed TOP_ATTACH to 7 and
+				 * BOTTOM_ATTACH to 8   06/04/1999
+				 */
+				gtk_table_attach (GTK_TABLE (table), ci->rec, i, i+1, 7, 8, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
+				gtk_widget_show (ci->rec);
+			} else { /* 
+				  * David: need to init it to null
+				  * otherwise we get a segfault when
+				  * trying to toggle
+				  * the buttons 
+				  */
+				ci->rec = NULL;
+			}
+	
+			ci->mute=gtk_check_button_new_with_label(_("Mute"));
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ci->mute), (ci->device->mute_bitmask & (1<<ci->channel))!=0);
+			gtk_signal_connect (GTK_OBJECT (ci->mute), "toggled", (GtkSignalFunc) mute_cb, (gpointer)ci);
+			gtk_table_attach (GTK_TABLE (table), ci->mute, i, i+1, 6, 7, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
+			gtk_widget_show (ci->mute);
+
+			separator = gtk_vseparator_new ();
+			/* BOTTOM_ATTACH changed to 8 */
+			gtk_table_attach (GTK_TABLE (table), separator, i+1, i+2, 1, 8, GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+			gtk_widget_show (separator);
+			i+=2; 
+		}
+	}
+
+}
+
+
 void open_dialog(void)
 {
-	GtkWidget *table, *notebook, *spixmap;
 	GList *d, *c;
 	
 	int i,j;
@@ -562,8 +815,8 @@ void open_dialog(void)
 	app = gnome_app_new ("gmix", _("GMIX 3.0") );
 	gtk_widget_realize (app);
 	gtk_signal_connect (GTK_OBJECT (app), "delete_event",
-		GTK_SIGNAL_FUNC (quit_cb),
-		NULL);
+			    GTK_SIGNAL_FUNC (quit_cb),
+			    NULL);
 
 	/*
 	 * Build main menue
@@ -583,92 +836,11 @@ void open_dialog(void)
 	/*
 	 * Build table with sliders
 	 */	
-	notebook = gtk_notebook_new();
-	gtk_widget_show(notebook);
-	gnome_app_set_contents(GNOME_APP (app), notebook);
+	slidernotebook = gtk_notebook_new();
+	gtk_widget_show(slidernotebook);
+	gnome_app_set_contents(GNOME_APP (app), slidernotebook);
 
-	i=0;
-	for (d=devices; d; d=d->next) {
-		device_info *di;
-		di=d->data;
-		j=0;
-		for (c=((device_info *)d->data)->channels;c;c=c->next) {
-			j+=2;
-		}
-
-		/* 
-		 * David: changed 7 to 8 for table rows (06/04/1999)
-		 */
-		table=gtk_table_new(i*2, 8, FALSE);
-		gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
-			table, 
-			gtk_label_new(di->info.name));
-		gtk_table_set_row_spacings (GTK_TABLE (table), 0);
-		gtk_table_set_col_spacings (GTK_TABLE (table), 0);
-		gtk_container_border_width (GTK_CONTAINER (table), 0);
-		gtk_widget_show (table);
-
-		for (c=((device_info *)d->data)->channels;c;c=c->next) {
-			GtkWidget *label, *mixer, *separator;
-			channel_info *ci;
-			ci=c->data;
-			if (ci->pixmap)
-			 {
-			  spixmap = gnome_stock_pixmap_widget (app, ci->pixmap);
-                          gtk_table_attach (GTK_TABLE (table), spixmap, i, i+1, 1, 2, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
-			  gtk_widget_show (spixmap);
-			 }
-			label=gtk_label_new(ci->title);
-			gtk_misc_set_alignment (GTK_MISC(label), 0.1, 0.5);
-			gtk_table_attach (GTK_TABLE (table), label, i, i+1, 2, 3, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
-			gtk_widget_show(label);
-
-			mixer=make_slider_mixer(ci);
-			gtk_table_attach (GTK_TABLE (table), mixer, i, i+1, 3, 4, GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
-			gtk_widget_show (mixer);
-
-			if (ci->device->stereodevs & (1<<ci->channel)) {
-				/* lock-button, only useful for stereo */
-				ci->lock = gtk_check_button_new_with_label(_("Lock"));
-				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ci->lock), (ci->device->lock_bitmask & (1<<ci->channel))!=0);
-				gtk_signal_connect (GTK_OBJECT (ci->lock), "toggled", (GtkSignalFunc) lock_cb, (gpointer)ci);
-				gtk_table_attach (GTK_TABLE (table), ci->lock, i, i+1, 4, 5, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
-				gtk_widget_show (ci->lock);
-			}
-	/*
-	 * recording sources
-	 */
-			if (ci->device->recmask & (1<<ci->channel)) {
-				ci->rec = gtk_check_button_new_with_label (_("Rec."));
-				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ci->rec), (ci->device->record_bitmask & (1<<ci->channel))!=0);
-				gtk_signal_connect (GTK_OBJECT (ci->rec), "toggled", (GtkSignalFunc) rec_cb, (gpointer)ci);
-				/* David: changed TOP_ATTACH to 7 and
-				 * BOTTOM_ATTACH to 8   06/04/1999
-				 */
-				gtk_table_attach (GTK_TABLE (table), ci->rec, i, i+1, 7, 8, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
-				gtk_widget_show (ci->rec);
-			} else { /* 
-				* David: need to init it to null
-				* otherwise we get a segfault when
-				* trying to toggle
-				* the buttons 
-				*/
-				ci->rec = NULL;
-			}
-	
-			ci->mute=gtk_check_button_new_with_label(_("Mute"));
-			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ci->mute), (ci->device->mute_bitmask & (1<<ci->channel))!=0);
-			gtk_signal_connect (GTK_OBJECT (ci->mute), "toggled", (GtkSignalFunc) mute_cb, (gpointer)ci);
-			gtk_table_attach (GTK_TABLE (table), ci->mute, i, i+1, 6, 7, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
-			gtk_widget_show (ci->mute);
-
-			separator = gtk_vseparator_new ();
-			/* BOTTOM_ATTACH changed to 8 */
-			gtk_table_attach (GTK_TABLE (table), separator, i+1, i+2, 1, 8, GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
-			gtk_widget_show (separator);
-			i+=2; 
-		}
-	}
+	fill_in_device_guis(slidernotebook);
 
 	gtk_widget_show(app);
 
@@ -700,7 +872,7 @@ void lock_cb (GtkWidget *widget, channel_info *data)
 
 void mute_cb (GtkWidget *widget, channel_info *data)
 {
-    unsigned long vol;
+	unsigned long vol;
 	if (data==NULL) return;
 	data->device->mute_bitmask&=~(1<<data->channel);
 	if (GTK_TOGGLE_BUTTON (data->mute)->active) {
@@ -748,7 +920,7 @@ void rec_cb(GtkWidget *widget, channel_info *data)
 
 void adj_left_cb (GtkAdjustment *adjustment, channel_info *data)
 {
-    unsigned long vol;
+	unsigned long vol;
 	if (data==NULL) return;
 	
 	if (data->device->stereodevs & (1<<data->channel)) {
@@ -776,7 +948,7 @@ void adj_left_cb (GtkAdjustment *adjustment, channel_info *data)
 
 void adj_right_cb (GtkAdjustment *adjustment, channel_info *data)
 {
-    unsigned long vol;
+	unsigned long vol;
 	if (data==NULL) return;
 	
 	if (data->device->stereodevs & (1<<data->channel)) {
@@ -810,9 +982,9 @@ void about_cb (GtkWidget *widget, void *data)
 		NULL
 	};
 	about = gnome_about_new ( _("GMIX - The Gnome Mixer"), VERSION,
-		"(C) 1998 Jens Ch. Restemeier",
-		(const gchar**)authors,
-		_("This is a mixer for OSS sound-devices."),
-		NULL);
+				  "(C) 1998 Jens Ch. Restemeier",
+				  (const gchar**)authors,
+				  _("This is a mixer for OSS sound-devices."),
+				  NULL);
 	gtk_widget_show (about);
 }
